@@ -7,7 +7,6 @@ lswifi.lswifi
 a CLI Wi-Fi scanning utility for Windows that leverages Microsofts Native Wifi wlanapi.h
 """
 
-import asyncio
 import logging
 import os
 import platform
@@ -36,7 +35,7 @@ from .constants import APNAMEACKFILE, APNAMEJSONFILE
 
 
 def app_path() -> None:
-    appdata_path = os.path.join(os.getenv("LOCALAPPDATA"), __title__)
+    appdata_path = os.path.join(os.getenv("LOCALAPPDATA"), __title__)  # type: ignore
     path_exists = os.path.isdir(appdata_path)
     if not path_exists:
         os.mkdir(appdata_path)
@@ -47,7 +46,8 @@ def main():
     is_apname_ack_stored = False
     parser = appsetup.setup_parser()
     args = parser.parse_args()
-    log = appsetup.setup_logger(args)
+    appsetup.setup_logger(args)
+    log = logging.getLogger(__name__)
     log.debug(f"args {args}")
     log.debug(f"{sys.version}")
     if args.data_location:
@@ -56,22 +56,17 @@ def main():
     if args.apnames:
         is_apname_ack_stored = user_ack_apnames_disclaimer()
         log.debug(
-            f"is there a stored ack for caching apnames on local machine? ({is_apname_ack_stored})"
+            f"is there a stored acknowledgement for caching apnames on local machine? {'Yes' if is_apname_ack_stored else 'No'}"
         )
 
-    try:
-        asyncio.run(core.scan(args, storedack=is_apname_ack_stored))
-    except KeyboardInterrupt:
-        log.warning("caught KeyboardInterrupt... stopping...")
-    except SystemExit:
-        pass
+    core.start(args, storedack=is_apname_ack_stored)
 
 
 def user_ack_apnames_disclaimer() -> bool:
     """retrieve ack from user that BSSIDs and discovered apnames will be cached in appdata"""
     logger = logging.getLogger(__name__)
     if os.getenv("LOCALAPPDATA"):
-        appdata_folder = os.path.join(os.getenv("LOCALAPPDATA"), __title__)
+        appdata_folder = os.path.join(os.getenv("LOCALAPPDATA"), __title__)  # type: ignore
     else:
         raise OSError
     is_path = os.path.isdir(appdata_folder)
@@ -85,16 +80,16 @@ def user_ack_apnames_disclaimer() -> bool:
     else:
         print(
             "---\n"
-            "AP (Access Point) names are not contained in every scan result.\n\n"
             "What?\n"
             "  - This feature locally caches BSSIDs and any detected corresponding AP names.\n"
             "  - Caching this information helps to more consistently provide AP names in output.\n"
             "Why?\n"
             "  - AP names are typically identified in beacon frames.\n"
-            "  - Dwell time varies per channel meaning it could be less than the beacon interval.\n"
-            "  - Retrieved scan results are a combination of beacons, probe responses, or sometimes a merged frame.\n"
+            "  - Dwell time (during a scan) varies per channel and can be shorter than the standard beacon interval.\n"
+            "  - Retrieved scan results are a combination of beacons, probe responses, and sometimes a merge of both.\n"
+            "  - This means results for a given BSSID might be from a beacon in one scan and a probe response in another.\n"
             "Where?\n"
-            "  - Data is stored and read from a JSON file on your local device here:\n\n"
+            "  - Any data collected is stored and read from a JSON file on your local device here:\n\n"
             f"{apnames}\n"
             "---\n"
         )
