@@ -8,21 +8,107 @@ Provides init functions that are used to help set up the app.
 """
 
 import argparse
+import datetime
 import logging
 import logging.config
+import os
+import sys
 import textwrap
+import time
+from pathlib import Path
 
 from .__version__ import __version__
 
 
 class ExportAction(argparse.Action):
+    """Enable export argument
+
+    Adds support for bool or bssid
+    """
+
     def __call__(self, parser, namespace, values, option_string=None):
+        """If no values, return something arbitrary so the arg is not None."""
         if not values:
-            values = 4
+            values = "all"
+        setattr(namespace, self.dest, values)
+
+
+BOOT_TIME = (
+    datetime.datetime.now()
+    .astimezone()
+    .replace(microsecond=0)
+    .isoformat()
+    .replace(":", "_")
+)
+
+
+def VerifyPath(_path, _extension):
+    # if absolute path is a directory and not a file
+    if os.path.isdir(_path):
+        print(f"{_path} is a directory not a file. exiting...")
+        sys.exit(-1)
+    # ensure we're writing to a file ending with .csv
+    if not _path.lower().endswith(_extension):
+        print(f"Exiting because file does not end with {_extension}")
+        sys.exit(-1)
+    # parent of the path exists?
+    if Path(_path).parent.absolute().exists():
+        # do we actually have write access?
+        try:
+            temp_file_path = (
+                Path(_path).parent.absolute()
+                / f"lswifi_write_test_{int(time.time() * 1000)}.tmp"
+            )
+            file_out = open(temp_file_path, "w")
+            file_out.close()
+            os.remove(temp_file_path)
+        except PermissionError:
+            if Path(_path).is_absolute():
+                print(f"Problem with write permission to {_path}. exiting...")
+            else:
+                print(
+                    f"Problem with write permission in {Path(_path).parent.absolute()} for {_path}. exiting..."
+                )
+            sys.exit(-1)
+    # does the parent directory of the path provided actually exist?
+    if Path(_path).is_absolute():
+        if not Path(_path).parent.absolute().exists():
+            print(
+                f"Parent directory at {Path(_path).parent.absolute()} does not exist. exiting..."
+            )
+            sys.exit(-1)
+
+
+class WriteToCSVAction(argparse.Action):
+    """Enable write to file arguments
+
+    Intended for CSV and JSON options
+    """
+
+    def __call__(self, parser, namespace, values, option_string=None):
+        """If no values, return something arbitrary so the arg is not None."""
+        if not values:
+            values = f"lswifi_{BOOT_TIME}.csv"
+        VerifyPath(values, ".csv")
+        setattr(namespace, self.dest, values)
+
+
+class WriteToJSONAction(argparse.Action):
+    """Enable write to file arguments
+
+    Intended for CSV and JSON options
+    """
+
+    def __call__(self, parser, namespace, values, option_string=None):
+        """If no values, return something arbitrary so the arg is not None."""
+        if not values:
+            values = f"lswifi_{BOOT_TIME}.json"
+        VerifyPath(values, ".json")
         setattr(namespace, self.dest, values)
 
 
 def setup_logger(args) -> logging.Logger:
+    """Set up the logger"""
     default_logging = {
         "version": 1,
         "disable_existing_loggers": False,
@@ -43,8 +129,48 @@ def setup_logger(args) -> logging.Logger:
     return logging.getLogger(__name__)
 
 
+def sensitivity(value):
+    """Validate user provided sensitivity is between -1 and -100"""
+    try:
+        display_sensitivity = int(value)
+        if display_sensitivity not in range(-100, -1):
+            raise argparse.ArgumentTypeError(
+                "rssi sensitivity threshold must be between -1 and -100"
+            )
+    except ValueError:
+        raise argparse.ArgumentTypeError(f"{value} not a valid threshold")
+    return display_sensitivity
+
+
+def json_indent(value):
+    """Validate user provided pretty print json value is sane and between 0 and 4"""
+    try:
+        if value is None:
+            return None
+        indent_level = int(value)
+        if indent_level not in range(0, 5):
+            raise argparse.ArgumentTypeError("JSON indent level must be less than 5")
+    except ValueError:
+        raise argparse.ArgumentTypeError(f"{value} not a valid JSON indent level")
+    return indent_level
+
+
+def width(value):
+    """Validate user provided channel bandwidth is 20, 40, 80, or 160"""
+    valid_channels = ["20", "40", "80", "160"]
+    if value == "None":
+        return None
+
+    width = str(value)
+    if width not in valid_channels:
+        raise argparse.ArgumentTypeError(
+            f"channel width {width} not valid. must use one of these: {', '.join(valid_channels)}"
+        )
+    return width
+
+
 def setup_parser() -> argparse.ArgumentParser:
-    """Setup the parser for arguments passed into the module from the CLI.
+    """Sets up the parser for arguments passed into the module from the CLI.
 
     Returns:
         argparse object.
@@ -79,7 +205,7 @@ def setup_parser() -> argparse.ArgumentParser:
             Print only networks with BSSIDs that contain <mac>:
               >lswifi -bssid 06:6D:15:88:81:59
 
-            Print additional details (inc. information elements) for a particular BSSID <mac>:
+            Print additional details (inc. information elements) for a provided BSSID:
               >lswifi -ies 06:6D:15:88:81:59
               
             Print and add detected AP names column in output:
@@ -90,14 +216,40 @@ def setup_parser() -> argparse.ArgumentParser:
 
             Watch event notifications (inc. roaming, connection, scanning, etc.):
               >lswifi --watchevents
-             """
+"""
         ),
-        epilog=f"Made with Python by Josh Schmelzle",
+        epilog="Made with Python by Josh Schmelzle",
         fromfile_prefix_chars="@",
     )
-    parser.add_argument("-version", action="version", version=f"%(prog)s {__version__}")
     parser.add_argument(
-        "-iface", dest="iface", metavar="INTERFACE", help="set which interface to use"
+        "-version",
+        "--version",
+        "-V",
+        action="version",
+        version=f"%(prog)s {__version__}",
+    )
+    # parser.add_argument(
+    #     "-iface", dest="iface", metavar="INTERFACE", help="set which interface to use", help=argparse.SUPPRESS
+    # )
+    parser.add_argument(
+        "-n",
+        "--scans",
+        dest="scans",
+        metavar="#",
+        help="set how many scans to do before exiting",
+    )
+    parser.add_argument(
+        "--time",
+        dest="time",
+        metavar="#",
+        help="set test in seconds to perform scans for",
+    )
+    parser.add_argument(
+        "-i",
+        "--interval",
+        dest="interval",
+        metavar="#",
+        help="seconds between scans",
     )
     parser.add_argument(
         "-ies",
@@ -109,22 +261,30 @@ def setup_parser() -> argparse.ArgumentParser:
     parser.add_argument(
         "-threshold",
         "-t",
-        metavar="dBm",
+        metavar="-82",
         dest="sensitivity",
-        help="threshold which excludes networks with weak signal strength (-82 is default)",
+        default="-82",
+        type=sensitivity,
+        help="threshold which excludes networks with weak signal strength from results (-82 is default)",
     )
     parser.add_argument(
         "-g",
+        # "-2",  # numbers as args interfere with -t arg
+        dest="g",
         action="store_true",
         help="display filter to limit output by 2.4 GHz band",
     )
     parser.add_argument(
         "-a",
+        # "-5",  # numbers as args interfere with -t arg
+        dest="a",
         action="store_true",
         help="display filter to limit output by 5 GHz band",
     )
     parser.add_argument(
         "-six",
+        # "-6",  # numbers as args interfere with -t arg
+        dest="six",
         action="store_true",
         help="display filter to limit output by 6 GHz band",
     )
@@ -172,14 +332,14 @@ def setup_parser() -> argparse.ArgumentParser:
         help="adds Protected Management Frame column to output using information from AP beacon RSNE",
     )
     parser.add_argument(
-        "--interval",
-        dest="interval",
+        "--period",
+        dest="period",
         action="store_true",
         help="adds beacon period column to output using information from AP beacon",
     )
     parser.add_argument(
-        "-uptime",
         "--uptime",
+        "-uptime",
         dest="uptime",
         action="store_true",
         help="sort output by access point uptime based on beacon timestamp",
@@ -187,6 +347,8 @@ def setup_parser() -> argparse.ArgumentParser:
     parser.add_argument(
         "--channel-width",
         dest="width",
+        type=width,
+        default="None",
         metavar="20|40|80|160",
         help="display filter to limit output by a specified channel width",
     )
@@ -249,10 +411,28 @@ def setup_parser() -> argparse.ArgumentParser:
     )
     parser.add_argument(
         "--json",
+        nargs="?",
+        type=str,
         dest="json",
-        action="store_true",
+        action=WriteToJSONAction,
         help="output will be formatted as json",
     )
+    parser.add_argument(
+        "--indent",
+        dest="json_indent",
+        metavar="4",
+        default=None,
+        type=json_indent,
+        help="JSON output will be formatted with pretty print with provided indent level",
+    )
+    parser.add_argument(
+        "--csv",
+        nargs="?",
+        type=str,
+        dest="csv",
+        action=WriteToCSVAction,
+        help="output will be formatted as csv",
+    )
     parser.add_argument(
         "-export",
         nargs="?",
@@ -260,7 +440,7 @@ def setup_parser() -> argparse.ArgumentParser:
         metavar="BSSID",
         dest="export",
         action=ExportAction,
-        help="export bss and ies bytefiles. default behavior will export all scan. to export one, specify mac address as argument.",
+        help="export bss and ies bytefiles. default behavior will export all from a scan. to export only one, provide full mac address of the BSSID as argument.",
     )
     parser.add_argument(
         "-decode",
@@ -272,7 +452,7 @@ def setup_parser() -> argparse.ArgumentParser:
         "--bytes",
         metavar="BSSID",
         dest="bytes",
-        help="output debugging bytes for a specified BSSID.",
+        help="output debugging bytes for a specified BSSID found in scan results.",
     )
     parser.add_argument(
         "--watchevents",
