@@ -7,12 +7,14 @@ lswifi.client
 client side code for requesting a scan, waiting for scan complete, and getting the results.
 """
 
+import datetime
 import functools
 import logging
+import os
 import pprint
+import sys
 import traceback
-from subprocess import check_output
-from threading import Timer
+from threading import Lock, Timer
 from types import SimpleNamespace
 from typing import Union
 
@@ -35,7 +37,78 @@ from .wlanapi import (
 )
 
 
+class TimerEx(object):
+    """
+    A reusable thread safe timer implementation
+    """
+
+    def __init__(self, interval_sec, function, *args, **kwargs):
+        """
+        Create a timer object which can be restarted
+
+        :param interval_sec: The timer interval in seconds
+        :param function: The user function timer should call once elapsed
+        :param args: The user function arguments array (optional)
+        :param kwargs: The user function named arguments (optional)
+        """
+        self._interval_sec = interval_sec
+        self._function = function
+        self._args = args
+        self._kwargs = kwargs
+        # Locking is needed since the '_timer' object might be replaced in a different thread
+        self._timer_lock = Lock()
+        self._timer = None
+
+    def start(self, restart_if_alive=True):
+        """
+        Starts the timer and returns this object [e.g. my_timer = TimerEx(10, my_func).start()]
+
+        :param restart_if_alive: 'True' to start a new timer if current one is still alive
+        :return: This timer object (i.e. self)
+        """
+        with self._timer_lock:
+            # Current timer still running
+            if self._timer is not None:
+                if not restart_if_alive:
+                    # Keep the current timer
+                    return self
+                # Cancel the current timer
+                self._timer.cancel()
+            # Create new timer
+            self._timer = Timer(self._interval_sec, self.__internal_call)
+            self._timer.start()
+        # Return this object to allow single line timer start
+        return self
+
+    def cancel(self):
+        """
+        Cancels the current timer if alive
+        """
+        with self._timer_lock:
+            if self._timer is not None:
+                self._timer.cancel()
+                self._timer = None
+
+    def is_alive(self):
+        """
+        :return: True if current timer is alive (i.e not elapsed yet)
+        """
+        with self._timer_lock:
+            if self._timer is not None:
+                return self._timer.is_alive()
+        return False
+
+    def __internal_call(self):
+        # Release timer object
+        with self._timer_lock:
+            self._timer = None
+        # Call the user defined function
+        self._function(*self._args, **self._kwargs)
+
+
 class Event(object):
+    """Native Wifi event class"""
+
     ns_type_to_codes_dict = {
         WLAN_NOTIFICATION_SOURCE_NONE: None,
         WLAN_NOTIFICATION_SOURCE_ONEX: ONEX_NOTIFICATION_TYPE_ENUM,
@@ -77,7 +150,7 @@ class Event(object):
                     actual.InterfaceGuid,
                 )
                 return event
-            except:
+            except Exception:
                 return None
 
     def __str__(self):
@@ -109,21 +182,20 @@ def parse_result(result, data_type, **kwargs):
 
 
 def get_interface_info(args, iface) -> str:
-    logging.getLogger(__name__)
     outstr = ""
     interface_info = {}
 
     # query interface for supported info
     params = ["current_connection", "channel_number", "statistics", "rssi"]
-    for p in params:
-        result = WLAN_API.WLAN.query_interface(iface, p)
-        interface_info[p] = result
+    for param in params:
+        result = WLAN_API.WLAN.query_interface(iface, param)
+        interface_info[param] = result
 
         if args.supported:
             if isinstance(result, tuple):
-                outstr += f"    {p}: {pprint.pformat(result, indent=4)}\n"
+                outstr += f"    {param}: {pprint.pformat(result, indent=4)}\n"
             else:
-                outstr += f"    {p}: {result}\n"
+                outstr += f"    {param}: {result}\n"
             return outstr
 
     isState = None
@@ -137,9 +209,9 @@ def get_interface_info(args, iface) -> str:
 
     if not args.get_current_channel and not args.get_current_ap:
         if connected:
-            outstr += f"Interface: {iface.description} is connected\n"
+            outstr += f"Interface: {iface.connection_name} is connected\n"
         else:
-            outstr += f"Interface: {iface.description} is disconnected\n"
+            outstr += f"Interface: {iface.connection_name} is disconnected\n"
 
     if not args.supported:
         # print(interface_info.items())
@@ -243,57 +315,115 @@ def get_interface_info(args, iface) -> str:
                 if key == "channel_number":
                     channel = result[0].value
 
-                    outstr += "    Channel: {}\n".format(channel)
+                    outstr += "    Channel: {0}\n".format(channel)
 
                     if args.get_current_ap and args.get_current_channel:
                         if args.raw:
                             return f"{bssid}, {channel}"
                         else:
-                            return f"Interface: {iface.description}, BSSID: {bssid}, CHANNEL: {channel}"
+                            return f"Interface: {iface.connection_name}, MAC: {iface.mac}, BSSID: {bssid}, CHANNEL: {channel}"
 
                     if args.get_current_ap:
                         if args.raw:
                             return bssid
                         else:
-                            return f"Interface: {iface.description}, BSSID: {bssid}"
+                            return f"Interface: {iface.connection_name}, MAC: {iface.mac}, BSSID: {bssid}"
 
                     if args.get_current_channel:
                         if args.raw:
                             return channel
                         else:
-                            return f"Interface: {iface.description}, Channel: {channel}"
+                            return f"Interface: {iface.connection_name}, MAC: {iface.mac}. Channel: {channel}"
         return outstr
     else:
         return ""
 
 
 class Client(object):
-    scan_finished = False
-    data = None
-    guid = ""
-    mac = ""
-    log = logging.getLogger(__name__)
-    get_bssid_args = SimpleNamespace(
-        get_current_ap=True,
-        raw=True,
-        event_watcher=True,
-        get_current_channel=False,
-        supported=False,
-    )
+    def __init__(self, args, iface, ssid=None):
+        try:
+            self.log = logging.getLogger(__name__)
+            self.scan_finished = False
+            self.data = None
+            now = datetime.datetime
+            self.last_scan_time_epoch = now.utcnow().timestamp()
+            self.last_scan_time_iso = (
+                now.now().astimezone().isoformat(timespec="milliseconds")
+            )
+            self.last_scan_time_utc = now.utcnow()
+            self.get_bssid_args = SimpleNamespace(
+                get_current_ap=True,
+                raw=True,
+                event_watcher=True,
+                get_current_channel=False,
+                supported=False,
+            )
+            self.timeout_interval = 4.0
+            self.client_handle = WLAN_API.WLAN.open_handle()
+            # self.scan_timer = Timer(self.timeout_interval, self.scan_timeout)
+            self.scan_timer = TimerEx(self.timeout_interval, self.scan_timeout)
+            self.args = args
+            self.iface = iface
+            self.mac = iface.mac
+            # self.first_event = True
+            self.is_handle_closed = False
+            self.callback = self.register_notification(
+                self.on_event_notification, self.client_handle
+            )
+            callbacks.append(self.callback)
+            self.log.debug("callback %s added", self.callback)
+            handles.append(self.client_handle)
+            self.log.debug("handle %s added", self.client_handle)
+        except Exception:
+            traceback.print_exc()
+            WLAN_API.WLAN.close_handle(self.client_handle)
+
+    def __del__(self):
+        # callbacks.remove(self.client_handle)
+        if not self.is_handle_closed:
+            result = WLAN_API.WLAN.close_handle(self.client_handle)
+            self.log.debug(
+                "handle %s closed with result %s", self.client_handle, result
+            )
+            if int(result) == 0:
+                self.is_handle_closed = True
+            else:
+                self.log.debug(
+                    "problem closing %s with result", self.client_handle, result
+                )
 
     def get_bss_list(self, interface) -> Union[list, None]:
         if interface:
-            wireless_network_bss_list = WLAN_API.WLAN.get_wireless_network_bss_list(
-                interface
-            )
-            if len(wireless_network_bss_list) == 0:
-                return None
+            try:
+                wireless_network_bss_list = WLAN_API.WLAN.get_wireless_network_bss_list(
+                    interface
+                )
 
-            return wireless_network_bss_list
+                if len(wireless_network_bss_list) == 0:
+                    return None
+
+                return wireless_network_bss_list
+            except Exception:
+                exception_type, exception_object, exception_traceback = sys.exc_info()
+                fname = os.path.split(exception_traceback.tb_frame.f_code.co_filename)[
+                    1
+                ]
+                self.log.error(
+                    "unexpected error when trying to get the BSS list on interface with MAC %s",
+                    interface.mac,
+                )
+                self.log.error(
+                    f"{exception_object} {exception_type} {fname}:{exception_traceback.tb_lineno}"
+                )
+                return None
         else:
             return None
 
-    def on_event_notification(self, wlan_event) -> None:
+    def on_event_notification(self, wlan_event, iface_guid) -> None:
+        if self.iface.guid_string == str(iface_guid):
+            pass
+        else:
+            return
         if wlan_event is not None:
             # attempt to get connected bssid
             bssid = get_interface_info(self.get_bssid_args, self.iface)
@@ -301,7 +431,6 @@ class Client(object):
             # if we want to watch wlan events on the terminal
             if self.args.event_watcher:
                 # if str(wlan_event).strip() in ["interface_removal", "interface_arrival"]:
-                #    self.mac = self.lookup_mac_on_guid(self.iface)
                 # if we want verbose info printed to the terminal
                 if self.args.debug:
                     if str(wlan_event).strip() in [
@@ -334,12 +463,17 @@ class Client(object):
 
                     self.log.debug(f"({self.mac}), start get_bss_list...")
                     self.data = self.get_bss_list(self.iface)
-                    self.log.debug(f"({self.mac}), finish get_bss_list...")
                     self.scan_finished = True
+                    now = datetime.datetime
+                    self.last_scan_time_epoch = now.utcnow().timestamp()
+                    self.last_scan_time_iso = (
+                        now.now().astimezone().isoformat(timespec="milliseconds")
+                    )
+                    self.last_scan_time_utc = now.utcnow()
+                    self.log.debug(f"({self.mac}), finish get_bss_list...")
 
                 # if str(wlan_event).strip() == "network_available":
                 #    pass
-                #
 
     def register_notification(self, callback, handle):
         c_back = WLAN_API.WLAN.wlan_register_notification(
@@ -351,13 +485,14 @@ class Client(object):
     def on_wlan_notification(callback, notification_data, p):
         event = Event.from_notification_data(notification_data)
         if event is not None:
-            callback(event)
+            callback(event, notification_data.contents.InterfaceGuid)
 
     def my_handle(self):
         return self.client_handle
 
     async def scan(self):
         try:
+            self.scan_finished = False
             self.log.debug(f"{self.iface.guid}: scan requested...")
             self.scan_timer.start()
             WLAN_API.WLAN.scan(self.iface.guid)
@@ -369,29 +504,6 @@ class Client(object):
                 scan_error,
             )
 
-    def lookup_mac_on_guid(self, iface) -> str:
-        guid = str(iface.guid)[1:-1]  # remove { } around guid
-        result = guid  # store guid in result
-        exe = 'getmac.exe /FO "CSV" /V'  # use getmac.exe to map interface guid to mac
-        cmd = f"{exe}"
-        try:
-            output = check_output(cmd)
-            mac = ""
-            self.log.debug(
-                "checking output from '%s' for client mac lookup on guid", exe
-            )
-            for line in output.decode().splitlines():
-                if guid in line or iface.description in line:
-                    conn = line.split(",")
-                    mac = conn[2].replace('"', "")
-                    result = mac.lower().replace("-", ":")
-                    self.log.debug(f"guid {guid} maps to {result}")
-                    break
-        except FileNotFoundError:
-            pass
-        finally:
-            return result
-
     def scan_timeout(self) -> None:
         """
         The application should then wait to receive the
@@ -399,28 +511,10 @@ class Client(object):
         Then the application can call the WlanGetNetworkBssList or WlanGetAvailableNetworkList
           function to retrieve a list of available wireless networks.
         """
-        self.log.info(f"timeout interval ({self.timeout_interval} seconds) exceeded...")
+        self.log.info(
+            f"timeout interval ({self.timeout_interval} seconds) for {self.mac} exceeded..."
+        )
         self.log.debug(f"({self.mac}), start get_bss_list...")
         self.data = self.get_bss_list(self.iface)
         self.log.debug(f"({self.mac}), finish get_bss_list...")
         self.scan_finished = True
-
-    def __init__(self, args, iface, ssid=None):
-        try:
-            self.timeout_interval = 4.0
-            self.scan_timer = Timer(self.timeout_interval, self.scan_timeout)
-            self.args = args
-            self.iface = iface
-            self.mac = self.lookup_mac_on_guid(iface)
-            # self.first_event = True
-            self.client_handle = WLAN_API.WLAN.open_handle()
-            callback = self.register_notification(
-                self.on_event_notification, self.client_handle
-            )
-            callbacks.append(callback)
-            self.log.debug(f"callback {callback} added")
-            handles.append(self.client_handle)
-            self.log.debug(f"handle {self.client_handle} added")
-        except Exception:
-            traceback.print_exc()
-            WLAN_API.WLAN.close_handle(self.client_handle)
