@@ -9,17 +9,21 @@ code to manage clients (interfaces), their data, and writing out results.
 
 # python imports
 import asyncio
+import concurrent.futures
 import contextlib
+import csv
 import datetime
 import json
 import logging
 import os
 import sys
+import time
+from operator import itemgetter
 from time import sleep
 
 # app imports
 from . import wlanapi as WLAN_API
-from .__version__ import __title__
+from .__version__ import __title__, __version__
 from .client import Client, get_interface_info
 from .constants import APNAMEJSONFILE
 from .elements import WirelessNetworkBss
@@ -37,50 +41,40 @@ from .helpers import (
 from .schemas.out import *
 
 
-def list_interfaces(interfaces) -> None:
-    """
-    Print interfaces and exit
-    """
-
-    print(f"There are {len(interfaces)} interfaces on this system:")
-    for interface in interfaces:
-        print(
-            f"    Description: {interface.description}\n"
-            f"    GUID: {interface.guid_string.replace('{', '').replace('}', '').lower()}\n"
-            f"    State: {interface.state_string}\n"
-        )
-    sys.exit()
-
-
-def watch_events(args, interfaces) -> None:
-    """
-    Watch for notifications on wireless interfaces
-    """
-    for interface in interfaces:
-        Client(args, interface)
-
-    try:
-        while True:
-            sleep(5)
-    except KeyboardInterrupt:
-        pass
-
-
-async def scan(args, **kwargs):
-    """
-    async func to perform a scan
-    """
-    clients = []
+def start(args, **kwargs):
     log = logging.getLogger(__name__)
-    interfaces = WLAN_API.WLAN.get_wireless_interfaces()
+    loops_completed = 0
+    is_caching_acknowledged = False
+
+    if kwargs is not None:
+        for key, value in kwargs.items():
+            if args.apnames and "stored" in key:
+                log.debug(
+                    f"has user provided ack for caching AP names on their local machine? {'Yes' if value else 'No'}"
+                )
+                if value:
+                    is_caching_acknowledged = value
 
     try:
+        clients = {}
+        for index, iface in WLAN_API.WLAN.get_wireless_interfaces().items():
+            clients[index] = Client(args, iface)
+
         if args.list_interfaces:
-            list_interfaces(interfaces)
+            print(f"There are {len(clients)} interfaces on this system:")
+            for _index, client in clients.items():
+                print(
+                    f"    Connection Name: {client.iface.connection_name}\n"
+                    f"    Description: {client.iface.description}\n"
+                    f"    GUID: {client.iface.guid_string.replace('{', '').replace('}', '').lower()}\n"
+                    f"    MAC: {client.iface.mac}\n"
+                    f"    State: {client.iface.state_string}\n"
+                )
+            sys.exit(0)
 
         if args.event_watcher:
-            watch_events(args, interfaces)
-            sys.exit(0)
+            while True:
+                sleep(5)
 
         if args.append:
             appendEthers(args.append)
@@ -90,43 +84,210 @@ async def scan(args, **kwargs):
             displayEthers()
             sys.exit(0)
 
-        for interface in interfaces:
+        if args.bytefile:
+            decode_bytefile(args)
+            sys.exit(0)
+
+        scanning = True
+
+        # for _index, interface in interfaces.items():
+        #     if (
+        #         args.get_interface_info
+        #         or args.get_current_ap
+        #         or args.get_current_channel
+        #         or args.supported
+        #     ):
+        #         scanning = False
+        #         print(get_interface_info(args, interface))
+
+        for _index, client in clients.items():
             if (
                 args.get_interface_info
                 or args.get_current_ap
                 or args.get_current_channel
                 or args.supported
             ):
-                print(get_interface_info(args, interface))
-                continue
-            elif args.bytefile:
-                decode_bytefile(args)
-                sys.exit(0)
-            else:  # TODO: this should be async.
-                client = Client(args, interface)
-                clients.append(client)
-                await client.scan()
+                scanning = False
+                print(get_interface_info(args, client.iface))
 
-        while True:  # can something be futured or awaited?
-            if all([i.scan_finished for i in clients]):
-                break
-            else:
-                await asyncio.sleep(0.1)
+        # TODO move other non-scanning functions here
 
-        for client in clients:
-            if client.data is not None:
-                log.debug(f"start parsing bss ies for {client.mac}")
-                parse_bss_list_and_print(client.data, args, **kwargs)
-                log.debug(f"finish parsing bss ies for {client.mac}")
-        sys.exit(0)
+        if scanning:
+            scans = 1
+            interval = 0.1
+            timeout = 0
+            if args.interval:
+                interval = int(args.interval)
+                log.debug(f"interval between scans is {interval}")
+            if args.scans:
+                scans = int(args.scans)
+                log.debug(f"number of scans requested is {scans}")
+            if args.time:
+                timeout = int(args.time)
+                log.debug(f"duration of time for recurring scans is {timeout} seconds")
+
+            csv_file_name = ""
+            json_file_name = ""
+            if args.csv or args.json:
+                csv_file_name = args.csv
+                json_file_name = args.json
+
+            if timeout > 0:  # we're scanning during a given time period
+                timeout_start = time.time()
+
+                while time.time() < timeout_start + timeout:
+                    asyncio.run(
+                        scan(
+                            clients,
+                            is_caching_acknowledged,
+                            csv_file_name,
+                            json_file_name,
+                            args,
+                        )
+                    )
+                    loops_completed += 1
+                    time.sleep(interval)
+            else:  # we're scanning a given number of times
+                _first = True
+                for _index in range(scans):
+                    if _first:
+                        _first = False
+                    else:
+                        time.sleep(interval)
+                    asyncio.run(
+                        scan(
+                            clients,
+                            is_caching_acknowledged,
+                            csv_file_name,
+                            json_file_name,
+                            args,
+                        )
+                    )
+                    loops_completed += 1
+
+            if loops_completed > 1:
+                log.info(f"total number of completed scans is {loops_completed}")
+    except KeyboardInterrupt:
+        if not args.event_watcher:
+            if loops_completed > 1:
+                log.info(
+                    f"total number of completed scans during this session is {loops_completed}"
+                )
+        log.warning("keyboard interruption detected... stopping...")
+        sys.exit(-1)
     except asyncio.CancelledError:
-        pass
+        raise
     except SystemExit as error:
         if error == 0:
             log.error(error)
         else:
             pass
 
+    sys.exit(0)
+
+
+async def scan(clients, is_caching_acknowledged, csv_file_name, json_file_name, args):
+    """
+    async func to perform a scan
+    """
+    log = logging.getLogger(__name__)
+    try:
+        background_tasks = set()
+        iface_count = len(clients)
+        if iface_count > 1:
+            log.info(f"starting scans on {iface_count} interfaces")
+
+        async def scanfunc(index, client):
+            log.debug(
+                f"initializing scan on {client.iface.description} {client.iface.guid_string}"
+            )
+            await client.scan()
+            log.debug(
+                f"initialized scan on {client.iface.description} {client.iface.guid_string}"
+            )
+            try:
+                while not client.scan_finished:
+                    time.sleep(0.1)
+            except Exception:
+                raise
+
+            clients[index] = client
+
+        for _index, client in clients.items():
+            task = scanfunc(_index, client)
+            background_tasks.add(task)
+
+        if len(clients.items()) > 0:
+            with concurrent.futures.ThreadPoolExecutor(
+                max_workers=len(clients.items())
+            ) as executor:
+                futures = []
+                for task in background_tasks:
+                    futures.append(executor.submit(asyncio.run, task))
+                for _future in concurrent.futures.as_completed(futures):
+                    pass
+
+        clients = {
+            k: clients[k] for k in sorted(clients)
+        }  # sort by key (index) numerically
+
+        for _idx, client in clients.items():
+            if client.data is None:
+                log.warning(f"no scan data for {client.mac}")
+            else:
+                log.debug(f"start parsing information elements for {client.mac}")
+                (
+                    out_results,
+                    bss_len,
+                    bssid_list,
+                    json_names,
+                    json_out,
+                    newapnames,
+                ) = parse_bss_list(
+                    client, is_caching_acknowledged, csv_file_name, json_file_name, args
+                )
+
+                if args.ies or args.bytes or args.export:
+                    return
+                print_bss_list(
+                    out_results,
+                    bss_len,
+                    client.mac,
+                    bssid_list,
+                    is_caching_acknowledged,
+                    json_names,
+                    json_out,
+                    newapnames,
+                    args,
+                )
+                log.debug(f"finish parsing information elements for {client.mac}")
+
+        ############################
+        # old sync example working #
+        ############################
+        #
+        # for _index, interface in interfaces.items():
+        #     client = Client(args, interface)
+        #     clients[_index] = client
+        #     log.debug(
+        #         f"initializing scan on {client.iface.description} {client.iface.guid_string}"
+        #     )
+        #     await client.scan()
+        #     log.debug(
+        #         f"initialized scan on {client.iface.description} {client.iface.guid_string}"
+        #     )
+        #     while not client.scan_finished:
+        #         pass
+        #     if client.data is None:
+        #         log.debug(f"no scan data for {client.mac}")
+        #     else:
+        #         log.debug(f"start parsing bss ies for {client.mac}")
+        #         parse_bss_list_and_print(client.data, client, args, **kwargs)
+        #         log.debug(f"finish parsing bss ies for {client.mac}")
+
+    except KeyboardInterrupt:
+        raise
+
 
 def displayEthers():
     log = logging.getLogger(__name__)
@@ -144,7 +305,7 @@ def displayEthers():
                 content = file_reader.readlines()
             content = [x.strip() for x in content]
             print("\n".join(content))
-    except:
+    except Exception:
         pass
 
 
@@ -193,8 +354,7 @@ def appendEthers(data):
 
 
 def loadEthers() -> dict:
-    logging.getLogger(__name__)
-    appdata_path = os.path.join(os.getenv("LOCALAPPDATA"), __title__)
+    appdata_path = os.path.join(os.getenv("LOCALAPPDATA"), __title__)  # type: ignore
     is_path = os.path.isdir(appdata_path)
     if not is_path:
         os.makedirs(appdata_path)
@@ -212,7 +372,7 @@ def loadEthers() -> dict:
 
 def loadAPNames() -> dict:
     log = logging.getLogger(__name__)
-    appdata_path = os.path.join(os.getenv("LOCALAPPDATA"), __title__)
+    appdata_path = os.path.join(os.getenv("LOCALAPPDATA"), __title__)  # type: ignore
     is_path = os.path.isdir(appdata_path)
     if not is_path:
         os.makedirs(appdata_path)
@@ -231,7 +391,7 @@ def loadAPNames() -> dict:
 
 def updateAPNames(json_names, scan_names) -> None:
     log = logging.getLogger(__name__)
-    appdata_path = os.path.join(os.getenv("LOCALAPPDATA"), __title__)
+    appdata_path = os.path.join(os.getenv("LOCALAPPDATA"), __title__)  # type: ignore
     is_path = os.path.isdir(appdata_path)
     if not is_path:
         os.makedirs(appdata_path)
@@ -269,30 +429,25 @@ def updateAPNames(json_names, scan_names) -> None:
         log.debug("<updateAPNames> nothing to update")
 
 
-def parse_bss_list_and_print(wireless_network_bss_list, args, **kwargs):
-    DISPLAY_SENSITIVITY = -82
+def parse_bss_list(
+    client, is_caching_acknowledged, csv_file_name, json_file_name, args
+):
     out_results = []
     bssid_list = []
+    csv_out = []
+    json_out = []
+
+    wireless_network_bss_list = client.data
 
     log = logging.getLogger(__name__)
 
-    if kwargs is not None:
-        for key, value in kwargs.items():
-            if args.apnames and "stored" in key:
-                log.debug(
-                    f"has user provided ack for caching AP names on their local machine? ({value})"
-                )
-                if value:
-                    stored_ack = value
-                else:
-                    stored_ack = value
-
     if args.ethers:
         ethers = loadEthers()
 
-    if args.apnames:
-        if stored_ack:
-            json_names = loadAPNames()
+    json_names = {}
+
+    if is_caching_acknowledged:
+        json_names = loadAPNames()
 
     exportpath = None
 
@@ -321,8 +476,6 @@ def parse_bss_list_and_print(wireless_network_bss_list, args, **kwargs):
 
     newapnames = {}
 
-    json_out = []
-
     bss_len = len(wireless_network_bss_list)
     # WirelessNetworkBss object
     for index, bss in enumerate(wireless_network_bss_list):
@@ -335,7 +488,7 @@ def parse_bss_list_and_print(wireless_network_bss_list, args, **kwargs):
                 user_bss = args.bytes.lower()
 
             if args.export:
-                if args.export != 4:
+                if args.export != "all":  # if lswifi -export xx:xx:xx:nn:nn:nn
                     user_bss = args.export
                     # print(f"{bss_len} {index}")
                     # print(f"{wlanapi_bss} {user_bss}")
@@ -348,6 +501,7 @@ def parse_bss_list_and_print(wireless_network_bss_list, args, **kwargs):
                             )
                         continue
 
+                # if lswifi -export
                 export_bss = str(bss.bssid).lower().replace(":", "-")
 
                 bsspath = export_bss + ".bss"
@@ -371,12 +525,17 @@ def parse_bss_list_and_print(wireless_network_bss_list, args, **kwargs):
                     iesfile.close()
 
                 # print(f"{bsspath} {iespath}")
-
-                if args.export != 4:
+                if args.export != "all":
+                    log.info(
+                        f"found and exporting requested bssid from the scan results of {client.mac}."
+                    )
                     print(f"raw byte files for {args.export} exported to {exportpath}")
                     break
-                elif bss_len == index:
-                    print(f"{bss_len} total raw byte files exported to {exportpath}")
+                elif (bss_len - 1) == index:
+                    log.info(
+                        f"found and exporting {bss_len} bssids from the scan results of {client.mac}."
+                    )
+                    print(f"files exported to {exportpath}")
 
                 continue
 
@@ -385,6 +544,9 @@ def parse_bss_list_and_print(wireless_network_bss_list, args, **kwargs):
                 # print("{} {}".format(wlanapi_bss, user_bss))
                 continue
             if args.ies:
+                log.info(
+                    f"found requested bssid in the scan results from {client.mac}."
+                )
                 print(bss)
             if args.bytes:
                 print("bss.bssbytes.send():")
@@ -399,76 +561,63 @@ def parse_bss_list_and_print(wireless_network_bss_list, args, **kwargs):
             break
 
         # handle weakest rssi value we want to see displayed to the screen
-        if args.sensitivity:
-            try:
-                DISPLAY_SENSITIVITY = int(args.sensitivity)
-                if DISPLAY_SENSITIVITY not in range(-100, -1):
-                    print("rssi threshold must be between -1 and -100... exiting...")
-                    sys.exit(-1)
-            except ValueError:
-                print(f"{args.sensitivity} not a valid threshold... exiting...")
-                sys.exit(-1)
-        if bss.rssi.value < DISPLAY_SENSITIVITY:
+        if bss.rssi.value < args.sensitivity:
             continue
 
+        # print(bss.ssid)
+
         # handle band filters
         if not args.a and not args.g and not args.six:
             pass
         else:
             # handle a band filter
             if args.a and args.g and not args.six:
-                if is_two_four_band(int(bss.channel_frequency.value)):
+                if is_two_four_band(bss.channel_frequency.value):
                     pass
-                if is_five_band(int(bss.channel_frequency.value)):
+                if is_five_band(bss.channel_frequency.value):
                     pass
-                if is_six_band(int(bss.channel_frequency.value)):
+                if is_six_band(bss.channel_frequency.value):
                     continue
             if args.a and args.six and not args.g:
-                if is_two_four_band(int(bss.channel_frequency.value)):
+                if is_two_four_band(bss.channel_frequency.value):
                     continue
-                if is_five_band(int(bss.channel_frequency.value)):
+                if is_five_band(bss.channel_frequency.value):
                     pass
-                if is_six_band(int(bss.channel_frequency.value)):
+                if is_six_band(bss.channel_frequency.value):
                     pass
             if args.a and not args.six and not args.g:
-                if is_two_four_band(int(bss.channel_frequency.value)):
+                if is_two_four_band(bss.channel_frequency.value):
                     continue
-                if is_five_band(int(bss.channel_frequency.value)):
+                if is_five_band(bss.channel_frequency.value):
                     pass
-                if is_six_band(int(bss.channel_frequency.value)):
+                if is_six_band(bss.channel_frequency.value):
                     continue
             # handle g band filter
             if args.g and args.six and not args.a:
-                if is_two_four_band(int(bss.channel_frequency.value)):
+                if is_two_four_band(bss.channel_frequency.value):
                     pass
-                if is_five_band(int(bss.channel_frequency.value)):
+                if is_five_band(bss.channel_frequency.value):
                     continue
-                if is_six_band(int(bss.channel_frequency.value)):
+                if is_six_band(bss.channel_frequency.value):
                     pass
             if args.g and not args.six and not args.a:
-                if is_two_four_band(int(bss.channel_frequency.value)):
+                if is_two_four_band(bss.channel_frequency.value):
                     pass
-                if is_five_band(int(bss.channel_frequency.value)):
+                if is_five_band(bss.channel_frequency.value):
                     continue
-                if is_six_band(int(bss.channel_frequency.value)):
+                if is_six_band(bss.channel_frequency.value):
                     continue
             # handle six band filter
             if args.six and not args.a and not args.g:
-                if is_two_four_band(int(bss.channel_frequency.value)):
+                if is_two_four_band(bss.channel_frequency.value):
                     continue
-                if is_five_band(int(bss.channel_frequency.value)):
+                if is_five_band(bss.channel_frequency.value):
                     continue
-                if is_six_band(int(bss.channel_frequency.value)):
+                if is_six_band(bss.channel_frequency.value):
                     pass
 
         # handle width filter
-        valid_channels = ["20", "40", "80", "160"]
         if args.width is not None:
-            if args.width not in valid_channels:
-                print(
-                    f"channel width {args.width} not valid. must use one of these values: {', '.join(valid_channels)}"
-                )
-                sys.exit(-1)
             if args.width not in str(bss.channel_width):
                 continue
 
@@ -504,7 +653,7 @@ def parse_bss_list_and_print(wireless_network_bss_list, args, **kwargs):
             if bss.bssid.value in ethers:
                 bss.apname.value = ethers[bss.bssid.value]
         elif args.apnames:
-            if stored_ack:
+            if is_caching_acknowledged:
                 scan_bssid = bss.bssid.value
                 scan_apname = remove_control_chars(bss.apname.value)
 
@@ -520,7 +669,7 @@ def parse_bss_list_and_print(wireless_network_bss_list, args, **kwargs):
                             ] = scan_apname  # then 1) update new hash table with current AP name
                             bss.apname.value = scan_apname  # then 2) update the apname that will be displayed
                     log.debug(
-                        f"LIVE BSSID {scan_bssid} CACHED {cachedAP} SCANNED {scan_apname}"
+                        f"BSSID from scan: {scan_bssid}, Name from cache: {cachedAP}, Name from scanned {scan_apname}"
                     )
                 elif scan_apname != "":  # working with new AP name
                     newapnames[
@@ -530,72 +679,94 @@ def parse_bss_list_and_print(wireless_network_bss_list, args, **kwargs):
         # bss.element.out() contains a tuple with the following values
         #   1. value, 2. header and alignment (left, center, right), 3. subheader
 
-        connected = False
         if bss.bssid.connected:
-            connected = True
-            if not args.json:
+            if not args.json and not args.csv:
+                # if "(*)" not in bss.bssid.value:
                 bss.bssid.value += "(*)"
 
-        json_out.append(
-            {
-                "amendments": sorted(bss.amendments.elements),
-                "apname": str(bss.apname).strip(),
-                "bssid": str(bss.bssid).strip(),
-                "bss_type=": str(bss.bss_type).strip(),
-                "channel_frequency": str(bss.channel_frequency).strip(),
-                "channel_number": str(bss.channel_number).strip(),
-                "channel_width": str(bss.channel_width).strip(),
-                "connected": connected,
-                "ies": sorted(bss.ie_numbers.elements),
-                "ies_extension": sorted(bss.exie_numbers.elements),
-                "modes": sorted(bss.modes.elements),
-                "phy_type": str(bss.phy_type).strip(),
-                "rates_basic": [x for x in bss.wlanrateset.basic.split(" ")],
-                "rates_data": [x for x in bss.wlanrateset.data.split(" ")],
-                "rssi": str(bss.rssi),
-                "security": str(bss.security).strip(),
-                "pmf": str(bss.pmf).strip(),
-                "spatial_streams": str(bss.spatial_streams),
-                "ssid": str(bss.ssid).strip(),
-                "stations": str(bss.stations),
-                "uptime": str(bss.uptime).strip(),
-                "utilization": str(bss.utilization).strip(),
-            }
+        if args.json:
+            json_out.append(
+                {
+                    "timestamp": client.last_scan_time_iso,
+                    "interface_mac": client.mac,
+                    "amendments": sorted(bss.amendments.elements),
+                    "apname": str(bss.apname).strip(),
+                    "bssid": str(bss.bssid).strip(),
+                    "bss_type": str(bss.bss_type).strip(),
+                    "channel_frequency": str(bss.channel_frequency).strip(),
+                    "channel_number": str(bss.channel_number).strip(),
+                    "channel_width": str(bss.channel_width).strip(),
+                    "connected": bss.bssid.connected,
+                    "ies": sorted(bss.ie_numbers.elements),
+                    "ies_extension": sorted(bss.exie_numbers.elements),
+                    "modes": sorted(bss.modes.elements),
+                    "pmf": str(bss.pmf).strip(),
+                    "phy_type": str(bss.phy_type).strip(),
+                    "rates_basic": [x for x in bss.wlanrateset.basic.split(" ")],
+                    "rates_data": [x for x in bss.wlanrateset.data.split(" ")],
+                    "rssi": str(bss.rssi),
+                    "security": str(bss.security).strip(),
+                    "spatial_streams": str(bss.spatial_streams),
+                    "ssid": str(bss.ssid),
+                    "stations": str(bss.stations),
+                    "uptime": str(bss.uptime).strip(),
+                    "utilization": str(bss.utilization).strip(),
+                }
+            )
+        if args.csv:
+            csv_out.append(
+                {
+                    "timestamp": client.last_scan_time_iso,
+                    "interface_mac": client.mac,
+                    "amendments": "/".join(sorted(bss.amendments.elements)),
+                    "apname": str(bss.apname).strip(),
+                    "bssid": str(bss.bssid).strip(),
+                    "bss_type": str(bss.bss_type).strip(),
+                    "channel_frequency": str(bss.channel_frequency).strip(),
+                    "channel_number": str(bss.channel_number).strip(),
+                    "channel_width": str(bss.channel_width).strip(),
+                    "connected": bss.bssid.connected,
+                    "ies": "/".join(map(str, sorted(bss.ie_numbers.elements))),
+                    "ies_extension": "/".join(
+                        map(str, sorted(bss.exie_numbers.elements))
+                    ),
+                    "modes": "/".join(sorted(bss.modes.elements)),
+                    "pmf": str(bss.pmf).strip(),
+                    "phy_type": str(bss.phy_type).strip(),
+                    "rates_basic": "/".join(
+                        [x for x in bss.wlanrateset.basic.split(" ")]
+                    ),
+                    "rates_data": "/".join(
+                        [x for x in bss.wlanrateset.data.split(" ")]
+                    ),
+                    "rssi": str(bss.rssi),
+                    "security": str(bss.security).strip(),
+                    "spatial_streams": str(bss.spatial_streams),
+                    "ssid": bss.ssid,
+                    "stations": str(bss.stations),
+                    "utilization": str(bss.utilization).strip(),
+                    "uptime": str(bss.uptime).strip(),
+                }
+            )
+        out_results.append(
+            [
+                bss.ssid.out(),
+                bss.bssid.out(),
+                bss.rssi.out(),
+                bss.phy_type.out(),
+                bss.channel_number_marked.out(),
+                bss.channel_frequency.out(),
+                bss.spatial_streams.out(),
+                bss.security.out(),
+                bss.amendments.out(),
+                bss.uptime.out(),
+            ]
         )
 
         if args.pmf:
-            out_results.append(
-                [
-                    bss.ssid.out(),
-                    bss.bssid.out(),
-                    bss.rssi.out(),
-                    bss.phy_type.out(),
-                    bss.channel_number_marked.out(),
-                    bss.channel_frequency.out(),
-                    bss.spatial_streams.out(),
-                    bss.security.out(),
-                    bss.amendments.out(),
-                    bss.pmf.out(),
-                    bss.uptime.out(),
-                ]
-            )
-        else:
-            out_results.append(
-                [
-                    bss.ssid.out(),
-                    bss.bssid.out(),
-                    bss.rssi.out(),
-                    bss.phy_type.out(),
-                    bss.channel_number_marked.out(),
-                    bss.channel_frequency.out(),
-                    bss.spatial_streams.out(),
-                    bss.security.out(),
-                    bss.amendments.out(),
-                    bss.uptime.out(),
-                ]
-            )
+            out_results[-1].insert(len(out_results[-1]) - 1, bss.pmf.out())
 
-        if args.interval:
+        if args.period:
             out_results[-1].append(bss.beacon_interval.out())
 
         if args.tpc:
@@ -606,7 +777,8 @@ def parse_bss_list_and_print(wireless_network_bss_list, args, **kwargs):
             out_results[-1].append(bss.utilization.out())
 
         if args.apnames or args.ethers:
-            out_results[-1].append(bss.apname.out())
+            if is_caching_acknowledged:
+                out_results[-1].append(bss.apname.out())
 
     def get_index(key):
         for r in out_results:
@@ -621,19 +793,86 @@ def parse_bss_list_and_print(wireless_network_bss_list, args, **kwargs):
             key=lambda x: int(x[get_index("UPTIME")].value.split("d")[0]),
             reverse=False,
         )
+        csv_out = sorted(csv_out, key=itemgetter("uptime"), reverse=False)
+        json_out = sorted(json_out, key=itemgetter("uptime"), reverse=False)
     else:  # sort by RSSI
         out_results = sorted(
             out_results, key=lambda x: x[get_index("RSSI")].value, reverse=False
         )
+        csv_out = sorted(csv_out, key=itemgetter("rssi"), reverse=False)
+        json_out = sorted(json_out, key=itemgetter("rssi"), reverse=False)
 
-    # here because i added the verbose, byte file func and export func to this func
-    if args.ies or args.bytes or args.export:
-        return
+    if args.json:
+        json_file_exists = os.path.exists(json_file_name)
+        if json_file_exists:
+            mode = "r+"
+        else:
+            mode = "w"
+        if not json_out:
+            log.info(f"nothing found to export as JSON to {json_file_name}")
+        else:
+            log.info(f"exporting scans as JSON to {json_file_name}")
+            with open(json_file_name, mode) as file:
+                if json_file_exists:
+                    file_data = json.load(file)
+                    file_data["scan_data"].append(json_out)
+                    file.seek(0)
+                    json.dump(file_data, file, indent=args.json_indent)
+                else:
+                    file_data = json.dumps(
+                        {
+                            "lswifi": {"version": f"lswifi {__version__}"},
+                            "scan_data": json_out,
+                        },
+                        indent=args.json_indent,
+                    )
+                    file.write(file_data)
 
-    # outlist to screen
+    if args.csv:
+        has_headings = False
+        csv_file_exists = os.path.exists(csv_file_name)
+        mode = "w"
+        if csv_file_exists:
+            mode = "a"
+            with open(csv_file_name, "r") as f:
+                try:
+                    has_headings = csv.Sniffer().has_header(f.read(4096))
+                except csv.Error:
+                    # The file seems to be empty
+                    has_headings = False
+
+        if not csv_out:
+            log.info(f"nothing found to export as CSV to {csv_file_name}")
+        else:
+            log.info(f"exporting scans as CSV to {csv_file_name}")
+            with open(csv_file_name, mode, newline="") as csvfile:
+                fieldnames = csv_out[-1].keys()
+                writer = csv.DictWriter(csvfile, fieldnames=fieldnames)
+                # log.debug("does csv file likely already have headings written? %s" % ('Yes' if has_headings else 'No'))
+                if not has_headings:
+                    writer.writeheader()
+                    has_headings = True
+                for result in csv_out:
+                    writer.writerow(result)
+
+    return out_results, bss_len, bssid_list, json_names, json_out, newapnames
+
+
+def print_bss_list(
+    out_results,
+    bss_len,
+    client_mac,
+    bssid_list,
+    is_caching_acknowledged,
+    json_names,
+    json_out,
+    newapnames,
+    args,
+):
+    log = logging.getLogger(__name__)
     log.info(
-        f"display filter sensitivity {DISPLAY_SENSITIVITY}; "
-        f"output includes {len(out_results)} of {len(wireless_network_bss_list)} BSSIDs detected in scan results."
+        f"display filter sensitivity {args.sensitivity}; "
+        f"output includes {len(out_results)} of {bss_len} BSSIDs detected in scan results for {client_mac}."
     )
 
     if len(out_results) > 0:
@@ -715,7 +954,7 @@ def parse_bss_list_and_print(wireless_network_bss_list, args, **kwargs):
         log.warning("***BSSIDS WITH DUPLICATE MACs***")
 
     if args.apnames:
-        if stored_ack:
+        if is_caching_acknowledged:
             updateAPNames(json_names, newapnames)
 
 
