@@ -4,32 +4,23 @@
 lswifi.libpcap
 ~~~~~~~~~~~~~~
 
-define libpcap class to help export to pcap
+define libpcap class to help export scan results to pcap file.
+
+https://gitlab.com/wireshark/wireshark/-/wikis/Development/LibpcapFileFormat
+
+perhaps in future pcapng. 
+
+inspiration and code extracted from https://github.com/secdev/scapy/blob/master/scapy/libs/winpcapy.py
 """
 
-
+# stdlib imports
 import logging
-
-# python imports
 import struct
 import time
+from ctypes import *
 
-# app imports
-
-""" global header
-This header starts the libpcap file and will be followed by the first packet header:
-
-typedef struct pcap_hdr_s {
-        guint32 magic_number;   /* magic number */
-        guint16 version_major;  /* major version number */
-        guint16 version_minor;  /* minor version number */
-        gint32  thiszone;       /* GMT to local correction */
-        guint32 sigfigs;        /* accuracy of timestamps */
-        guint32 snaplen;        /* max length of captured packets, in octets */
-        guint32 network;        /* data link type */
-} pcap_hdr_t;
-
-struct format characters
+""" FORMAT CHARACTERS FOR STRUCT
+================================
 
 | format | C Type             | Python type | standard size |
 | ------ | ------------------ | ----------- | ------------- |
@@ -48,27 +39,54 @@ byte order, size, and alignment
 | <         | little-endian | standard | none      |
 | >         | big-endian    | standard | none      |
 | !         | network       | standard | none      |
-
 """
 
-LIBPCAP_GLOBAL_HEADER_FMT = "@ I H H i I I I "
+# misc
+u_short = c_ushort
+bpf_int32 = c_int
+u_int = c_int
+bpf_u_int32 = u_int
+pcap = c_void_p
+pcap_dumper = c_void_p
+u_char = c_ubyte
+FILE = c_void_p
+STRING = c_char_p
 
-LIBPCAP_MAGIC_NUMBER = 2712847316  # hex 0xA1B2C3D4
-LIBPCAP_VERSION_MAJOR = 2
-LIBPCAP_VERSION_MINOR = 4
-LIBPCAP_THISZONE = 0
-LIBPCAP_SIGFIGS = 0
-LIBPCAP_SNAPLEN = 65535
-LIBPCAP_LINK_LAYER_HEADER_TYPE = 127
 
-LIBPCAP_RECORD_HEADER_FMT = "@ I I I I"
+class timeval(Structure):
+    _fields_ = [("tv_sec", c_long), ("tv_usec", c_long)]
 
-# LINK-LAYER HEADER TYPE VALUES:
-# LINKTYPE_ETHERNET             , 1  , IEEE 802.3 Ethernet
-# LINKTYPE_IEEE802_11           , 105, IEEE 802.11
-# LINKTYPE_IEEE802_11_RADIOTAP  , 127, Radiotap link-layer information followed by an 802.11 header
 
-""" Record (Packet) Header
+""" GLOBAL HEADER
+=================
+This header starts the libpcap file and will be followed by the first packet header:
+
+typedef struct pcap_hdr_s {
+        guint32 magic_number;   /* magic number */
+        guint16 version_major;  /* major version number */
+        guint16 version_minor;  /* minor version number */
+        gint32  thiszone;       /* GMT to local correction */
+        guint32 sigfigs;        /* accuracy of timestamps */
+        guint32 snaplen;        /* max length of captured packets, in octets */
+        guint32 network;        /* data link type */
+} pcap_hdr_t;
+"""
+
+
+class pcap_global_header(Structure):
+    _fields_ = [
+        ("magic_number", bpf_u_int32),
+        ("version_major", u_short),
+        ("version_minor", u_short),
+        ("thiszone", bpf_int32),
+        ("sigfigs", bpf_u_int32),
+        ("snaplen", bpf_u_int32),
+        ("linktype", bpf_u_int32),
+    ]
+
+
+""" RECORD (PACKET) HEADER
+==========================
 Each captured packet starts with (any byte alignment possible):
 
 typedef struct pcaprec_hdr_s {
@@ -80,10 +98,100 @@ typedef struct pcaprec_hdr_s {
 """
 
 
+class pcap_packet_header(Structure):
+    """
+    Header of a packet in the pcap file.
+    """
+
+    _fields_ = [("ts", timeval), ("caplen", bpf_u_int32), ("len", bpf_u_int32)]
+
+
+LIBPCAP_GLOBAL_HEADER_FMT = "@ I H H i I I I "
+
+"""
+Magic Number (32 bits):  an unsigned magic number, whose value is
+    either the hexadecimal number 0xA1B2C3D4 or the hexadecimal number
+    0xA1B23C4D.
+
+    If the value is 0xA1B2C3D4, time stamps in Packet Records (see
+    Figure 2) are in seconds and microseconds; if it is 0xA1B23C4D,
+    time stamps in Packet Records are in seconds and nanoseconds.
+
+Used to detect the file format itself and the byte ordering. 
+
+    The writing application writes 0xa1b2c3d4 with it's native byte 
+    ordering format into this field. 
+
+    The reading application will read either 0xa1b2c3d4 (identical)
+    or 0xd4c3b2a1 (swapped).
+
+    If the reading application reads the swapped 0xd4c3b2a1 value, 
+    it knows that all the following fields will have to be swapped too.
+"""
+
+LIBPCAP_MAGIC_NUMBER = 2712847316  # hex 0xA1B2C3D4
+
+LIBPCAP_VERSION_MAJOR = 2
+LIBPCAP_VERSION_MINOR = 4
+
+"""
+If the timestamps are in GMT (UTC), thiszone is simply 0.
+time stamps should always be GMT. so, this is 0.
+"""
+LIBPCAP_THISZONE = 0
+
+"""
+sigfigs: in theory, the accuracy of time stamps in the capture;
+in practice, all tools set it to 0
+"""
+LIBPCAP_SIGFIGS = 0
+LIBPCAP_SNAPLEN = 65535
+
+
+class LINKTYPE(object):
+    """
+    LINK-LAYER HEADER TYPE VALUES
+    https://www.tcpdump.org/linktypes.html
+    """
+
+    def __init__(self, name, value, dlt):
+        self.name = name
+        self.value = value
+        self.dlt = dlt
+
+
+LINKTYPE_IEEE802_11 = LINKTYPE("LINKTYPE_IEEE802_11", 105, "DLT_IEEE802_11")
+LINKTYPE_IEEE802_11_RADIOTAP = LINKTYPE(
+    "LINKTYPE_IEEE802_11_RADIOTAP", 127, "DLT_IEEE802_11_RADIO"
+)
+
+LIBPCAP_RECORD_HEADER_FMT = "@ I I I I"
+
+# LINK-LAYER HEADER TYPE VALUES:
+# LINKTYPE_ETHERNET             , 1  , IEEE 802.3 Ethernet
+# LINKTYPE_IEEE802_11           , 105, IEEE 802.11
+# LINKTYPE_IEEE802_11_RADIOTAP  , 127, Radiotap link-layer information followed by an 802.11 header
+
+
+""" PACKET DATA
+The actual packet data will immediately follow the packet header as a data blob of incl_len bytes without a specific byte alignment.
+"""
+
+
 class LIBPCAP:
-    def __init__(self, file, header_type=LIBPCAP_LINK_LAYER_HEADER_TYPE):
-        log = logging.getLogger(__name__)
-        self.pcap = open(file, "wb")
+    def __init__(self, file, header_type, data):
+        self.log = logging.getLogger(__name__)
+        self.file = file
+        self.header_type = header_type
+        self.data = data
+        self.pcap = None
+
+    def write(self):
+        self.pcap = open(self.file, "wb")
+        self.writerheader()
+        self.writerdata()
+
+    def write_global_header(self):
         self.pcap.write(
             struct.pack(
                 LIBPCAP_GLOBAL_HEADER_FMT,
@@ -93,24 +201,39 @@ class LIBPCAP:
                 LIBPCAP_THISZONE,
                 LIBPCAP_SIGFIGS,
                 LIBPCAP_SNAPLEN,
-                header_type,
+                self.header_type,
             )
         )
-        log.debug(f"[+] Header Type: {header_type}")
+        self.log.debug(f"[+] Header Type: {header_type}")
+
+    def write_record_header(self):
+        """
+        Each captured packet starts with (any byte alignment possible):
+        """
+        self.pcap.write(
+            struct.pack(
+                self.header_type,
+                int(time.time()),
+                int(time.time() * 1000000),
+                len(self.data),
+                len(self.data),
+            )
+        )
+        self.log.debug(f"[+] Record Header: {self.data}")
+
+    def write_packet_data(self):
+        """
+        The actual packet data will immediately follow the packet header
+        as a data blob of incl_len bytes without a specific byte alignment.
+        """
+        self.pcap.write(self.data)
+        self.log.debug(f"[+] Packet Data: {self.data}")
 
     def writelist(self, data=[]):
         for i in data:
             self.write(i)
         return
 
-    def write(self, data):
-        ts_sec, ts_usec = map(int, str(time.time()).split("."))
-        length = len(data)
-        self.pcap.write(
-            struct.pack(LIBPCAP_RECORD_HEADER_FMT, ts_sec, ts_usec, length, length)
-        )
-        self.pcap.write(data)
-
     def close(self):
         self.pcap.close()
 
@@ -149,12 +272,3 @@ RADIOTAP_HEADER_FMT = RADIOTAP_HEADER_FMT + "I b"
 class RADIOTAP:
     def __init__(self, channel, rssi):
         return struct.pack(RADIO)
-
-
-""" IEEE header
-TODO: fill in. 
-"""
-
-
-class IEEE80211:
-    pass
